@@ -29,6 +29,7 @@
 #' @param path.lwd The thickness of the line used to draw the path. By default
 #'   this is drawn heavier to make them stand out.
 #' @param lwd The thickness of the lines used to draw the arena. Default is 1.
+#' @param ... Additional plotting parameters (passed to \code{\link[terra]{plot}}).
 #'
 #' @seealso \code{\link{calculate_metrics}}, and also
 #'   \code{\link{read_experiment}} for processing many tracks at once.
@@ -47,7 +48,7 @@
 #' @importFrom terra plot
 #'
 #' @export
-plot_path = function(metrics, title = NULL, quadrants = FALSE, highlight = TRUE, margins = c(0, 0, 3, 0), path.lwd = NA, lwd = 1) {
+plot_path = function(metrics, title = NULL, quadrants = FALSE, highlight = TRUE, margins = c(0, 0, 3, 0), path.lwd = NA, lwd = 1, ...) {
 	if(methods::is(metrics, 'rtrack_metrics')){
 		if(is.null(title)){title = metrics$id}
 		if(is.null(title)){title = ""} # Empty title string in case using metrics$id failed
@@ -58,7 +59,7 @@ plot_path = function(metrics, title = NULL, quadrants = FALSE, highlight = TRUE,
 		metrics$arena$zones = lapply(metrics$arena$zones, terra::unwrap) # Deserialise arena zones.
 		
 		if(metrics$arena$description$type == "mwm"){
-			terra::plot(metrics$arena$zones$pool, lwd = lwd, mar = margins, axes = FALSE, main = title, xpd = NA)
+			terra::plot(metrics$arena$zones$pool, lwd = lwd, mar = margins, axes = FALSE, main = title, ..., xpd = NA)
 			terra::plot(metrics$arena$zones$wall, lwd = path.lwd, col = "#C0E0FFFF", add = T, border = NA)
 			terra::plot(metrics$arena$zones$far.wall, lwd = path.lwd, col = "#C0E0FFA0", add = T, border = NA)
 			terra::plot(metrics$arena$zones$annulus, lwd = path.lwd, col = "#C0E0FF40", add = T, border = NA)
@@ -82,7 +83,7 @@ plot_path = function(metrics, title = NULL, quadrants = FALSE, highlight = TRUE,
 			if(!is.null(metrics$arena$zones$goal)) graphics::segments(metrics$path$x[1], metrics$path$y[1], metrics$arena$goal$x, metrics$arena$goal$y, lty = 3, lwd = path.lwd, col = "#F0A020F0")
 			terra::plot(metrics$arena$zones$pool, lwd = path.lwd, border = "#606060FF", add = T)
 		}else if(metrics$arena$description$type == "barnes"){
-			terra::plot(metrics$arena$zones$arena, lwd = lwd, mar = margins, axes = FALSE, main = title, xpd = NA)
+			terra::plot(metrics$arena$zones$arena, lwd = lwd, mar = margins, axes = FALSE, main = title, ..., xpd = NA)
 			terra::plot(metrics$arena$zones$annulus, lwd = path.lwd, col = "#C0E0FF80", add = T, border = NA)
 			terra::plot(metrics$arena$zones$centre, lwd = path.lwd, col = "#C0E0FF80", add = T, border = NA)
 			if(quadrants){
@@ -108,14 +109,14 @@ plot_path = function(metrics, title = NULL, quadrants = FALSE, highlight = TRUE,
 			if(!is.null(metrics$arena$zones$goal)) graphics::segments(metrics$path$x[1], metrics$path$y[1], metrics$arena$goal$x, metrics$arena$goal$y, lty = 3, lwd = path.lwd, col = "#F0A020F0")
 			terra::plot(metrics$arena$zones$arena, lwd = path.lwd, border = "#606060FF", add = T)
 		}else if(metrics$arena$description$type == "oft"){
-			terra::plot(metrics$arena$zones$field, lwd = lwd, mar = margins, axes = FALSE, xlim = c(-1.1, 1.1), ylim = c(-1.1, 1.1), main = title, xpd = NA)
+			terra::plot(metrics$arena$zones$field, lwd = lwd, mar = margins, axes = FALSE, xlim = c(-1.2, 1.2), ylim = c(-1.2, 1.2), main = title, ..., xpd = NA)
 			terra::plot(metrics$arena$zones$centre, col = "#C0E0FF80", add = T, border = NA)
 			terra::plot(metrics$arena$zones$wall, col = "#C0E0FF80", add = T, border = NA)
 			terra::plot(metrics$arena$zones$corner, col = "#C0E0FFFF", add = T, border = NA)
 			graphics::lines(metrics$path$x, metrics$path$y, lwd = path.lwd, col = "#000000FF", xpd = NA)
 			terra::plot(metrics$arena$zones$field, lwd = path.lwd, border = "#606060FF", add = T)
 		}else if(metrics$arena$description$type == "nor"){
-			terra::plot(metrics$arena$zones$field, lwd = lwd, mar = margins, axes = FALSE, xlim = c(-1.1, 1.1), ylim = c(-1.1, 1.1), main = title, xpd = NA)
+			terra::plot(metrics$arena$zones$field, lwd = lwd, mar = margins, axes = FALSE, xlim = c(-1.2, 1.2), ylim = c(-1.2, 1.2), main = title, ..., xpd = NA)
 			terra::plot(metrics$arena$zones$wall, col = "#C0E0FF80", add = T, border = NA)
 			terra::plot(metrics$arena$zones$corner, col = "#C0E0FFFF", add = T, border = NA)
 			if(!metrics$arena$novel$object.1){
@@ -133,7 +134,7 @@ plot_path = function(metrics, title = NULL, quadrants = FALSE, highlight = TRUE,
 			graphics::lines(metrics$path$x, metrics$path$y, lwd = path.lwd, col = "#000000FF", xpd = NA)
 			terra::plot(metrics$arena$zones$field, lwd = path.lwd, border = "#606060FF", add = T)
 		}else if(metrics$arena$description$type == "apa"){
-			terra::plot(metrics$arena$zones$arena, lwd = lwd, mar = margins, axes = FALSE, main = title, xpd = NA)
+			terra::plot(metrics$arena$zones$arena, lwd = lwd, mar = margins, axes = FALSE, main = title, ..., xpd = NA)
 			# Make an arc for the angle-from-aversive. If no aversive, then ignore (NOT from old.aversive).
 			if(!is.null(metrics$arena$zones$aversive.zone)){
 				midaversive = (((metrics$arena$aversive.zone$end.angle + 360) + (metrics$arena$aversive.zone$start.angle + 360)) / 2) %% 360
@@ -180,7 +181,7 @@ plot_path = function(metrics, title = NULL, quadrants = FALSE, highlight = TRUE,
 			if(!is.null(metrics$arena$zones$aversive.zone)) terra::plot(metrics$arena$zones$aversive.zone, col = "#D0A00080", add = T)
 			if(!is.null(metrics$arena$zones$old.aversive.zone)) terra::plot(metrics$arena$zones$old.aversive.zone, col = "#80808080", add = T)
 			graphics::lines(metrics$path$x, metrics$path$y, lwd = path.lwd, col = "#000000FF", xpd = NA)
-			terra::plot(metrics$arena$zones$arena, lwd = path.lwd, border = "#606060FF", add = T, main = title, xpd = NA)
+			terra::plot(metrics$arena$zones$arena, lwd = path.lwd, border = "#606060FF", add = T, xpd = NA)
 
 			# Add an arrow for direction of rotation.
 			lines(x = sin(deg2rad(-26:-64)) * 1.1, y = cos(deg2rad(-26:-64)) * 1.1, lwd = 1.5 * path.lwd, col = "#A0A0A0FF", xpd = NA)
